"""Error codes.

:author: Stefan Lehmann <stlm@posteo.de>
:license: MIT, see license file or https://opensource.org/licenses/MIT
:created on: 2018-06-11 18:15:53

"""
from .errorcodes_nc import NC_ERROR_CODES

ERROR_CODES = {
    0: "no error",
    1: "Internal error",
    2: "No real time",
    3: "Allocation locked - memory error",
    4: "Mailbox full – the ADS message could not be sent.",
    5: "Wrong HMSG",
    6: "Target port not found - ADS Server not started",
    7: "Target machine not found - Missing ADS routes",
    8: "Unknown command ID",
    9: "Invalid task ID",
    10: "No IO",
    11: "Unknown AMS command",
    12: "Win 32 error",
    13: "Port not connected",
    14: "Invalid AMS length",
    15: "Invalid AMS Net ID",
    16: "Installation level is too low –TwinCAT 2 license error",
    17: "No debug available",
    18: "Port disabled",
    19: "Port already connected",
    20: "AMS Sync Win32 error",
    21: "AMS Sync Timeout",
    22: "AMS Sync AMS error",
    23: "No index map for AMS Sync available",
    24: "Invalid AMS port",
    25: "No memory",
    26: "TCP send error",
    27: "Host unreachable",
    28: "Invalid AMS fragment",
    29: "TLS send error, secure ADS connection failed",
    30: "Access denied, secure ADS access denied",
    1280: "ROUTERERR_NOLOCKEDMEMORY",
    1281: "ROUTERERR_RESIZEMEMORY",
    1282: "ROUTERERR_MAILBOXFULL",
    1283: "ROUTERERR_DEBUGBOXFULL",
    1284: "ROUTERERR_UNKNOWNPORTTYPE",
    1285: "ROUTERERR_NOTINITIALIZED",
    1286: "ROUTERERR_PORTALREADYINUSE",
    1287: "ROUTERERR_NOTREGISTERED",
    1288: "ROUTERERR_NOMOREQUEUES",
    1289: "ROUTERERR_INVALIDPORT",
    1290: "ROUTERERR_NOTACTIVATED",
    1291: "ROUTERERR_FRAGMENTBOXFULL",
    1292: "ROUTERERR_FRAGMENTTIMEOUT",
    1293: "ROUTERERR_TOBEREMOVED",
    1792: "error class <device error>",
    1793: "Service is not supported by server",
    1794: "invalid index group",
    1795: "invalid index offset",
    1796: "reading/writing not permitted",
    1797: "parameter size not correct",
    1798: "invalid parameter value(s)",
    1799: "device is not in a ready state",
    1800: "device is busy",
    1801: "invalid context (must be in Windows)",
    1802: "out of memory",
    1803: "invalid parameter value(s)",
    1804: "not found (files, ...)",
    1805: "syntax error in command or file",
    1806: "objects do not match",
    1807: "object already exists",
    1808: "symbol not found",
    1809: "symbol version invalid. Onlinechange. Release handle and get a new one",
    1810: "server is in invalid state",
    1811: "AdsTransMode not supported",
    1812: "Notification handle is invalid. Onlinechange. Release handle and get a new one",  # noqa: E501
    1813: "Notification client not registered",
    1814: "no more notification handles",
    1815: "size for watch too big",
    1816: "device not initialized",
    1817: "device has a timeout",
    1818: "query interface failed",
    1819: "wrong interface required",
    1820: "class ID is invalid",
    1821: "object ID is invalid",
    1822: "request is pending",
    1823: "request is aborted",
    1824: "signal warning",
    1825: "invalid array index",
    1826: "symbol not active. Onlinechange. Release handle and get a new one",
    1827: "access denied",
    1828: "missing license. Activate license for TwinCAT 3 function",
    1836: "exception occurred during system start. Check each device transitions",
    1856: "Error class <client error>",
    1857: "invalid parameter at service",
    1858: "polling list is empty",
    1859: "var connection already in use",
    1860: "invoke ID in use",
    1861: "timeout elapsed",
    1862: "error in win32 subsystem",
    1863: "Invalid client timeout value",
    1864: "ads-port not opened",
    1872: "internal error in ads sync",
    1873: "hash table overflow",
    1874: "key not found in hash",
    1875: "no more symbols in cache",
    1876: "invalid response received",
    1877: "sync port is locked",
    4096: "RTERR_INTERNAL",
    4097: "RTERR_BADTIMERPERIODS",
    4098: "RTERR_INVALIDTASKPTR",
    4099: "RTERR_INVALIDSTACKPTR",
    4100: "RTERR_PRIOEXISTS",
    4101: "RTERR_NOMORETCB",
    4102: "RTERR_NOMORESEMAS",
    4103: "RTERR_NOMOREQUEUES",
    4104: "TwinCAT reserved.",
    4105: "TwinCAT reserved.",
    4106: "TwinCAT reserved.",
    4107: "TwinCAT reserved.",
    4108: "TwinCAT reserved.",
    4109: "RTERR_EXTIRQALREADYDEF",
    4110: "RTERR_EXTIRQNOTDEF",
    4111: "RTERR_EXTIRQINSTALLFAILED",
    4112: "RTERR_IRQLNOTLESSOREQUAL",
    4119: "RTERR_VMXNOTSUPPORTED",
    4120: "RTERR_VMXDISABLED",
    4121: "RTERR_VMXCONTROLSMISSING",
    4122: "RTERR_VMXENABLEFAILS",
    10060: "A socket operation was attempted to an unreachable host",
    10061: "A connection attempt failed because the connected party did not properly respond after a period of time, or established connection failed because connected host has failed to respond.",  # noqa: E501
    10065: "No connection could be made because the target machine actively refused it",
}

# Update the ERROR_CODES dictionary with NC_ERROR_CODES
ERROR_CODES.update(NC_ERROR_CODES)


"""The testserver run script.

:author: Roberto Roos
:license: MIT, see license file or https://opensource.org/licenses/MIT
:created on: 2021-04-09

"""

import argparse

from .testserver import AdsTestServer, AdvancedHandler, BasicHandler, AbstractHandler


def main() -> None:
    """Main function (keep variable out of global scope)"""

    parser = argparse.ArgumentParser(description='Run an ADS Testserver')
    parser.add_argument("--host", default="127.0.0.1", help="host IP, default: 127.0.0.1")
    parser.add_argument("-p", "--port", default=48898, help="binding port, default: 48898", type=int)
    parser.add_argument('--handler', choices=['basic', 'advanced'], default='advanced',
                        help="testserver handler, default: advanced")
    args = parser.parse_args()

    handler: AbstractHandler
    if args.handler == 'basic':
        handler = BasicHandler()
    else:
        handler = AdvancedHandler()

    server = AdsTestServer(
        ip_address=args.host,
        port=args.port,
        handler=handler
    )

    # noinspection PyBroadException
    try:
        print('Starting testserver...')
        server.start()
        print('Running testserver at {}:{}'.format(server.ip_address, server.port))
        server.join()
    except:
        server.close()

    print('Testserver closed')


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""Constants for the work with the ADS API.

:author: Stefan Lehmann <stlm@posteo.de>
:license: MIT, see license file or https://opensource.org/licenses/MIT
:created on: 2018-06-11 18:15:53

"""
from typing import Type, Dict, Callable, Union
from ctypes import (
    Array,
    c_bool,
    c_ubyte,
    c_int8,
    c_uint8,
    c_int16,
    c_uint16,
    c_int32,
    c_uint32,
    c_float,
    c_double,
    c_char,
    c_wchar,
    c_int64,
    c_uint64,
)

STRING_BUFFER: int = 1024
PLC_DEFAULT_STRING_SIZE: int = 80

MAX_ADS_SUB_COMMANDS: int = 500


class PLCTYPE_WSTRING:
    """Special dummy class for handling WSTRING."""


# plc data types:
PLCTYPE_BOOL = c_bool
PLCTYPE_BYTE = c_ubyte
PLCTYPE_DWORD = c_uint32
PLCTYPE_DINT = c_int32
PLCTYPE_INT = c_int16
PLCTYPE_LREAL = c_double
PLCTYPE_REAL = c_float
PLCTYPE_SINT = c_int8
PLCTYPE_STRING = c_char
PLCTYPE_TOD = c_int32
PLCTYPE_UBYTE = c_ubyte
PLCTYPE_UDINT = c_uint32
PLCTYPE_UINT = c_uint16
PLCTYPE_USINT = c_uint8
PLCTYPE_WORD = c_uint16
PLCTYPE_LINT = c_int64
PLCTYPE_ULINT = c_uint64

PLCTYPE_DATE = PLCTYPE_DWORD
PLCTYPE_DATE_AND_TIME = PLCTYPE_DWORD
PLCTYPE_DT = PLCTYPE_DWORD
PLCTYPE_TIME = PLCTYPE_DWORD

# Typing
PLCSimpleDataType = Union[
    PLCTYPE_BOOL,
    PLCTYPE_BYTE,
    PLCTYPE_DWORD,
    PLCTYPE_DINT,
    PLCTYPE_INT,
    PLCTYPE_LREAL,
    PLCTYPE_REAL,
    PLCTYPE_SINT,
    PLCTYPE_STRING,
    PLCTYPE_WSTRING,
    PLCTYPE_TOD,
    PLCTYPE_UBYTE,
    PLCTYPE_UDINT,
    PLCTYPE_UINT,
    PLCTYPE_USINT,
    PLCTYPE_WORD,
    PLCTYPE_LINT,
    PLCTYPE_ULINT,
    PLCTYPE_DATE,
    PLCTYPE_DATE_AND_TIME,
    PLCTYPE_DT,
    PLCTYPE_TIME,
]
PLCDataType = Union[Array, PLCSimpleDataType]

# Datatype unpacking values
DATATYPE_MAP: Dict[Type, str] = {
    PLCTYPE_BOOL: "<?",
    PLCTYPE_BYTE: "<B",
    PLCTYPE_DINT: "<i",
    PLCTYPE_DWORD: "<I",
    PLCTYPE_INT: "<h",
    PLCTYPE_LREAL: "<d",
    PLCTYPE_REAL: "<f",
    PLCTYPE_SINT: "<b",
    PLCTYPE_UDINT: "<I",
    PLCTYPE_UINT: "<H",
    PLCTYPE_USINT: "<B",
    PLCTYPE_LINT: "<q",
    PLCTYPE_ULINT: "<Q",
    PLCTYPE_WORD: "<H"
}

# Used in get_string_size.get_string_size()
PLC_STRING_TYPE_NAME = "STRING"

# ADS data types
ADST_VOID: int = 0
ADST_INT8: int = 16
ADST_UINT8: int = 17
ADST_INT16: int = 2
ADST_UINT16: int = 18
ADST_INT32: int = 3
ADST_UINT32: int = 19
ADST_INT64: int = 20
ADST_UINT64: int = 21
ADST_REAL32: int = 4
ADST_REAL64: int = 5
ADST_BIGTYPE: int= 65
ADST_STRING: int = 30
ADST_WSTRING: int = 31
ADST_REAL80: int = 32
ADST_BIT: int = 33
ADST_MAXTYPES: int = 34


ads_type_to_ctype = {
    # ADST_VOID
    ADST_INT8: PLCTYPE_BYTE,
    ADST_UINT8: PLCTYPE_UBYTE,
    ADST_INT16: PLCTYPE_INT,
    ADST_UINT16: PLCTYPE_UINT,
    ADST_INT32: PLCTYPE_DINT,
    ADST_UINT32: PLCTYPE_UDINT,
    ADST_INT64: PLCTYPE_LINT,
    ADST_UINT64: PLCTYPE_ULINT,
    ADST_REAL32: PLCTYPE_REAL,
    ADST_REAL64: PLCTYPE_LREAL,
    # ADST_BIGTYPE
    ADST_STRING: PLCTYPE_STRING,
    ADST_WSTRING: PLCTYPE_WSTRING,
    # ADST_REAL80
    ADST_BIT: PLCTYPE_BOOL,
}


def PLCTYPE_ARR_REAL(n: int) -> Type[Array]:
    """Return an array with n float values."""
    return c_float * n


def PLCTYPE_ARR_LREAL(n: int) -> Type[Array]:
    """Return an array with n double values."""
    return c_double * n


def PLCTYPE_ARR_BOOL(n: int) -> Type[Array]:
    """Return an array with n boolean values."""
    return c_bool * n


def PLCTYPE_ARR_INT(n: int) -> Type[Array]:
    """Return an array with n int16 values."""
    return c_int16 * n


def PLCTYPE_ARR_UINT(n: int) -> Type[Array]:
    """Return an array with n uint16 values."""
    return c_uint16 * n


def PLCTYPE_ARR_SHORT(n: int) -> Type[Array]:
    """Return an array with n short values."""
    return c_int16 * n


def PLCTYPE_ARR_USHORT(n: int) -> Type[Array]:
    """Return an array with n ushort values."""
    return c_uint16 * n


def PLCTYPE_ARR_DINT(n: int) -> Type[Array]:
    """Return an array with n int32 values."""
    return c_int32 * n


def PLCTYPE_ARR_UDINT(n: int) -> Type[Array]:
    """Return an array with n uint32 values."""
    return c_uint32 * n


def PLCTYPE_ARR_SINT(n: int) -> Type[Array]:
    """Return an array with n int8 values."""
    return c_int8 * n


def PLCTYPE_ARR_USINT(n: int) -> Type[Array]:
    """Return an array with n uint8 values."""
    return c_uint8 * n


# Map c-type array names to PLCTYPE_* arrays
PLC_ARRAY_MAP: Dict[str, Callable] = {
    'real': PLCTYPE_ARR_LREAL,  # LREAL, not REAL
    'boolean': PLCTYPE_ARR_BOOL,
    'int32': PLCTYPE_ARR_DINT,
    'uint32': PLCTYPE_ARR_UDINT,
    'int16': PLCTYPE_ARR_INT,
    'uint16': PLCTYPE_ARR_UINT,
    'int8': PLCTYPE_ARR_SINT,
    'uint8': PLCTYPE_ARR_USINT,
}


# Index Group
# READ_M - WRITE_M
INDEXGROUP_MEMORYBYTE = 0x4020  #: plc memory area (%M), offset means byte-offset
# READ_MX - WRITE_MX
INDEXGROUP_MEMORYBIT = (
    0x4021
)  #: plc memory area (%MX), offset means the bit address, calculatedb by bytenumber * 8 + bitnumber  # noqa: E501
# PLCADS_IGR_RMSIZE
INDEXGROUP_MEMORYSIZE = 0x4025  #: size of the memory area in bytes
# PLCADS_IGR_RWRB
INDEXGROUP_RETAIN = 0x4030  #: plc retain memory area, offset means byte-offset
# PLCADS_IGR_RRSIZE
INDEXGROUP_RETAINSIZE = 0x4035  #: size of the retain area in bytes
# PLCADS_IGR_RWDB
INDEXGROUP_DATA = 0x4040  #: data area, offset means byte-offset
# PLCADS_IGR_RDSIZE
INDEXGROUP_DATASIZE = 0x4045  #: size of the data area in bytes


ADSIGRP_SYMTAB = 0xF000
ADSIGRP_SYMNAME = 0xF001
ADSIGRP_SYMVAL = 0xF002

ADSIGRP_SYM_HNDBYNAME = 0xF003
ADSIGRP_SYM_VALBYNAME = 0xF004
ADSIGRP_SYM_VALBYHND = 0xF005
ADSIGRP_SYM_RELEASEHND = 0xF006
ADSIGRP_SYM_INFOBYNAME = 0xF007
ADSIGRP_SYM_VERSION = 0xF008
ADSIGRP_SYM_INFOBYNAMEEX = 0xF009

ADSIGRP_SYM_DOWNLOAD = 0xF00A
ADSIGRP_SYM_UPLOAD = 0xF00B
ADSIGRP_SYM_UPLOADINFO = 0xF00C
ADSIGRP_SYM_DOWNLOAD2 = 0xF00D
ADSIGRP_SYM_DT_UPLOAD = 0xF00E
ADSIGRP_SYM_UPLOADINFO2 = 0xF00F

ADSIGRP_SYMNOTE = 0xF010  #: notification of named handle
ADSIGRP_IOIMAGE_RWIB = 0xF020  #: read/write input byte(s)
ADSIGRP_IOIMAGE_RWIX = 0xF021  #: read/write input bit
ADSIGRP_IOIMAGE_RWOB = 0xF030  #: read/write output byte(s)
ADSIGRP_IOIMAGE_RWOX = 0xF031  #: read/write output bit
ADSIGRP_IOIMAGE_CLEARI = 0xF040  #: write inputs to null
ADSIGRP_IOIMAGE_CLEARO = 0xF050  #: write outputs to null

ADSIGRP_SUMUP_READ = 0xF080  #: ADS Sum Read Request
ADSIGRP_SUMUP_WRITE = 0xF081  #: ADS Sum Write Request

ADSIGRP_DEVICE_DATA = 0xF100  #: state, name, etc...
ADSIOFFS_DEVDATA_ADSSTATE = 0x0000  #: ads state of device
ADSIOFFS_DEVDATA_DEVSTATE = 0x0002  #: device state


# PORTS
PORT_LOGGER: int = 100
PORT_EVENTLOGGER: int = 110
PORT_REALTIME_CORE: int = 200  # Real-time core
PORT_IO: int = 300
PORT_SPECIALTASK1: int = 301
PORT_SPECIALTASK2: int = 302
PORT_NC: int = 500
PORT_SPS1: int = 801  # Twincat 2 Runtime system 1
PORT_SPS2: int = 811  # Twincat 2 Runtime system 2
PORT_SPS3: int = 821  # Twincat 2 Runtime system 3
PORT_SPS4: int = 831  # Twincat 2 Runtime system 4
PORT_TC2PLC1: int = PORT_SPS1
PORT_TC2PLC2: int = PORT_SPS2
PORT_TC2PLC3: int = PORT_SPS3
PORT_TC2PLC4: int = PORT_SPS4
PORT_TC3PLC1: int = 851  # Twincat 3 Runtime system 1
PORT_TC3PLC2: int = 852  # Twincat 3 Runtime system 2
PORT_TC3PLC3: int = 853  # Twincat 3 Runtime system 3
PORT_TC3PLC4: int = 854  # Twincat 3 Runtime system 4
PORT_NOCKE: int = 900
PORT_CAM: int = PORT_NOCKE
PORT_SYSTEMSERVICE: int = 10000
PORT_SCOPE: int = 14000
PORT_REMOTE_UDP: int = 48899

# ADSState-constants
ADSSTATE_INVALID: int = 0
ADSSTATE_IDLE: int = 1
ADSSTATE_RESET: int = 2
ADSSTATE_INIT: int = 3
ADSSTATE_START: int = 4
ADSSTATE_RUN: int = 5
ADSSTATE_STOP: int = 6
ADSSTATE_SAVECFG: int = 7
ADSSTATE_LOADCFG: int = 8
ADSSTATE_POWERFAILURE: int = 9
ADSSTATE_POWERGOOD: int = 10
ADSSTATE_ERROR: int = 11
ADSSTATE_SHUTDOWN: int = 12
ADSSTATE_SUSPEND: int = 13
ADSSTATE_RESUME: int = 14
ADSSTATE_CONFIG: int = 15  # system is in config mode
ADSSTATE_RECONFIG: int = 16  # system should restart in config mode

# ADSTransmode
ADSTRANS_NOTRANS: int = 0  #: no notifications
ADSTRANS_CLIENTCYCLE: int = 1
ADSTRANS_CLIENT1REQ: int = 2
ADSTRANS_SERVERCYCLE: int = 3  #: notify on a cyclic base
ADSTRANS_SERVERONCHA: int = 4  #: notify everytime the value changes

# symbol flags
ADSSYMBOLFLAG_PERSISTENT = 0x00000001
ADSSYMBOLFLAG_BITVALUE = 0x00000002
ADSSYMBOLFLAG_REFERENCETO = 0x0004
ADSSYMBOLFLAG_TYPEGUID = 0x0008
ADSSYMBOLFLAG_TCCOMIFACEPTR = 0x0010
ADSSYMBOLFLAG_READONLY = 0x0020
ADSSYMBOLFLAG_CONTEXTMASK = 0x0F00

# ADS Command IDs
ADSCOMMAND_INVALID = 0x00
ADSCOMMAND_READDEVICEINFO = 0x01
ADSCOMMAND_READ = 0x02
ADSCOMMAND_WRITE = 0x03
ADSCOMMAND_READSTATE = 0x04
ADSCOMMAND_WRITECTRL = 0x05
ADSCOMMAND_ADDDEVICENOTE = 0x06
ADSCOMMAND_DELDEVICENOTE = 0x07
ADSCOMMAND_DEVICENOTE = 0x08
ADSCOMMAND_READWRITE = 0x09

# STATE Flags
ADSSTATEFLAG_REQRESP = 0x0001
ADSSTATEFLAG_COMMAND = 0x0004



# -*- coding: utf-8 -*-
#
# Configuration file for the Sphinx documentation builder.
#
# This file does only contain a selection of the most common options. For a
# full list see the documentation:
# http://www.sphinx-doc.org/en/master/config

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = 'adstool manual'
copyright = '2015 - 2021, Beckhoff Automation GmbH & Co. KG'
author = 'Patrick Brünn'

# The short X.Y version
version = ''
# The full version, including alpha/beta/rc tags
release = ''


# -- General configuration ---------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = 'en'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = None


# -- Options for manual page output ------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('adstool', 'adstool', 'ads, adstool, tcadstool Documentation', [author], 1),
]
